#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

/*
 * Funkcija system_our() implementira istu funckcionalnost kao funkcija
 * system() iz standardne biblioteke.
 */
static int system_our (const char *command);

/* Funkcija status_print() stampa informacije o zavrsnom statusu procesa. */
static void status_print (int status);

/*
 * Funkcija error_fatal() ispisuje dati string, a onda i poruku o sistemskoj
 * greski i potom prekida program.
 */
static void error_fatal (char *message);

/*
 * Program demonstrira implementaciju system() funkcije.
 */
int
main (int argc, char **argv)
{
  int status;			/* Izlazni status. */

  /* Poziva se funkcija system_our() da izvrsi komandu date. */
  if ((status = system_our ("date")) < 0)
    error_fatal ("system_our date");
  status_print (status);

  /*
   * Poziva se funkcija system_our() da izvrsi nepostojecu komandu.
   */
  if ((status = system_our ("nosuchcommand")) < 0)
    error_fatal ("system_our nosuchcommand");
  status_print (status);

  /* Poziva se funkcija system_our() da izvrsi sekvencu komandi. */
  if ((status = system_our ("who; exit 42")) < 0)
    error_fatal ("system_our who; exit 42");
  status_print (status);

  /* Zavrsava se program. */
  exit (EXIT_SUCCESS);
}

static int
system_our (const char *command)
{
  pid_t pid;			/* Identifikator procesa. */
  int status;			/* Informacija o zavrsetku deteta procesa. */

  /* Proverava se da li je zadata komanda kao argument. */
  if (command == NULL)
    return 1;

  /* Kreira se dete proces. */
  if ((pid = fork ()) < 0)
    return -1;
  /*
   * Dete proces pokrece shell da izvrsi datu komandu. Ako pritom
   * dodje do greske, vraca se 127 kao vrednost izlaznog statusa.
   */
  else if (pid == 0)
    {
      execl ("/bin/sh", "sh", "-c", command, (char *) 0);
      _exit (127);
    }
  /*
   * Roditeljski proces ceka da se dete proces zavrsi. Ako pritom dodje do
   * greske, vraca se -1 kao vrednost izlaznog statusa; ako pak sve
   * prodje u redu, vraca se izlazni status koji je dobijen od deteta
   * procesa (tj. od shell-a).
   */
  else
    {
      if (waitpid (pid, &status, 0) < 0)
	return -1;
      return status;
    }
}

static void
status_print (int status)
{
  /*
   * U skladu sa zavrsnim statusom procesa, stampa se odgovarajuca
   * poruka.
   */
  if (WIFEXITED (status))
    printf ("normal termination, exit status = %d\n", WEXITSTATUS (status));
  else if (WIFSIGNALED (status))
    printf ("abnormal termination, signal number = %d\n", WTERMSIG (status));
  else if (WIFSTOPPED (status))
    printf ("child stopped, signal number = %d\n", WSTOPSIG (status));

  printf ("\n");
}

static void
error_fatal (char *message)
{
  perror (message);
  exit (EXIT_FAILURE);
}
