#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/times.h>

void error_fatal (char *additional_info, const char *message){
  printf ("%s\n", additional_info);
  perror (message);
  exit (EXIT_FAILURE);
}

/* Funkcija izvrsava program koji joj se prosledjuje kao argument
   i ispisuje vremena izvrsvanja tog programa. */
void command_execute (char *command){
  struct tms tms_start, tms_end;
  clock_t start, end;
  long ticks_per_second;
  
  /* Pre izvrsavanja komande, odredjuje se broj proteklih 
     tikova od nekog (ne znamo kog) trenutka u proslosti. */
  if ( (start = times (&tms_start)) < 0)
    error_fatal (command, "first times call");

  /* Izvrsava se komanda. */
  if (system (command) < 0)
    error_fatal (command, "system error");

  /* Posle izvrsavanja komande, odredjuje se broj proteklih 
     tikova od nekog (ne znamo kog) trenutka u proslosti. */
  if ( (end = times (&tms_end)) < 0)
    error_fatal (command, "second times call");

  /* Odredjuje se broj tikova u sekundi.*/
  if ( (ticks_per_second = sysconf (_SC_CLK_TCK)) <= 0)
    error_fatal (command, "sysconf error");

  /* Sva vremena dobijaju se kao razlika odgovarajucih zavrsnog i pocetnog vremena. */
  printf ("real : %g\n", (double)(end - start) / ticks_per_second);
  printf ("user : %g\n", (double)(tms_start.tms_utime - tms_end.tms_utime) / ticks_per_second);
  printf ("system : %g\n", (double)(tms_start.tms_stime - tms_end.tms_stime) / ticks_per_second);
  printf ("child user : %g\n", (double)(tms_start.tms_cutime - tms_end.tms_cutime) / ticks_per_second);
  printf ("child system : %g\n", (double)(tms_start.tms_cstime - tms_end.tms_cstime) / ticks_per_second);

}

int main(int argc, char **argv){
  int i;

  for (i=1; i<argc; i++)
    command_execute (argv[i]);

  command_execute ("sleep 2");
  
  return 0;
}

